package org.commonjava.tensor.data;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertThat;

import java.util.Iterator;
import java.util.List;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.SimpleLayout;
import org.apache.maven.graph.common.ref.ProjectVersionRef;
import org.apache.maven.graph.effective.EProjectGraph;
import org.apache.maven.graph.effective.EProjectRelationships;
import org.apache.maven.graph.effective.ref.EProjectKey;
import org.apache.maven.graph.effective.traverse.AncestryTraversal;
import org.commonjava.tensor.data.store.TensorDataStore;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public abstract class TensorDataStoreTCK
{

    protected TensorDataStore db;

    @BeforeClass
    public static void initLogging()
    {
        LogManager.getLoggerRepository()
                  .setThreshold( Level.DEBUG );

        LogManager.getRootLogger()
                  .addAppender( new ConsoleAppender( new SimpleLayout() ) );

        LogManager.getRootLogger()
                  .setLevel( Level.DEBUG );
    }

    @Before
    public void setup()
        throws Exception
    {
        db = doDbSetup();
    }

    protected abstract TensorDataStore doDbSetup()
        throws Exception;

    @Test
    public void storeRelationshipAndValidateContains()
        throws Exception
    {
        final ProjectVersionRef r = new ProjectVersionRef( "org.test", "root", "1" );
        final EProjectRelationships root = new EProjectRelationships.Builder( r ).build();

        db.storeRelationshipSet( root, true );

        assertThat( db.containsRelationshipSet( new EProjectKey( r ) ), equalTo( true ) );
    }

    @Test
    public void storeGraphAndRetrieveAncestry()
        throws Exception
    {
        final ProjectVersionRef r = new ProjectVersionRef( "org.test", "root", "1" );
        final ProjectVersionRef p = new ProjectVersionRef( "org.test", "parent", "1.0" );
        final ProjectVersionRef c = new ProjectVersionRef( "org.test", "child", "1.0" );

        final EProjectRelationships root = new EProjectRelationships.Builder( r ).build();
        final EProjectRelationships parent = new EProjectRelationships.Builder( p ).withParent( r )
                                                                                   .build();
        final EProjectRelationships child = new EProjectRelationships.Builder( c ).withParent( p )
                                                                                  .build();

        final EProjectGraph g = new EProjectGraph( child );
        g.add( parent );
        g.add( root );

        assertThat( g.isComplete(), equalTo( true ) );

        db.storeGraph( g, true );

        final EProjectGraph graph = db.getGraph( new EProjectKey( c ) );

        assertThat( graph.isComplete(), equalTo( true ) );

        final AncestryTraversal ancestryTraversal = new AncestryTraversal();
        graph.traverse( ancestryTraversal );

        final List<ProjectVersionRef> ancestry = ancestryTraversal.getAncestry();
        assertThat( ancestry, notNullValue() );
        assertThat( ancestry.size(), equalTo( 3 ) );

        final Iterator<ProjectVersionRef> iterator = ancestry.iterator();
        assertThat( iterator.next(), equalTo( c ) );
        assertThat( iterator.next(), equalTo( p ) );
        assertThat( iterator.next(), equalTo( r ) );
    }

    @Test
    public void storeProjectWithOneDependencyAndRetrieve()
        throws Exception
    {
        final EProjectRelationships rels =
            new EProjectRelationships.Builder( new ProjectVersionRef( "org.apache.maven", "maven-core", "3.0.3" ) ).withDependency( new ProjectVersionRef(
                                                                                                                                                           "org.apache.maven",
                                                                                                                                                           "maven-artifact",
                                                                                                                                                           "3.0.3" ),
                                                                                                                                    null,
                                                                                                                                    null,
                                                                                                                                    null,
                                                                                                                                    false )
                                                                                                                   .build();

        db.storeRelationshipSet( rels, true );

        final EProjectRelationships result = db.getRelationshipSet( new EProjectKey( rels.getProjectRef() ) );

        assertThat( result.getDependencies()
                          .size(), equalTo( 1 ) );
        assertThat( result.getDependencies()
                          .get( 0 ), equalTo( rels.getDependencies()
                                                  .get( 0 ) ) );
    }

}